#! /usr/bin/env python
#
# Semplice script di interrogazione di LDAP per creare
# le home degli utenti locali
#
# Copyright (C) 2005-2007 FUSS Project <info@fuss.bz.it>
# Author: Simone Piccardi <piccardi@truelite.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import ldap
from os import *
from sys import *
import re

flags  = re.IGNORECASE|re.MULTILINE

# Get LDAP parameters from standard files
try:
    regexp = r'^\s*base\s*(.*)'
    regx   = re.compile(regexp, flags)
    conf   = file("/etc/ldap/ldap.conf").read() 
    base   = regx.findall(conf)[0]
    regexp = r'^\s*uri\s*(.*)'
    regx   = re.compile(regexp, flags)
    uri    = regx.findall(conf)[0]
except Exception,e:
    print "cannot read configuration from /etc/ldap/ldap.conf"
    print e

# get LDAP password
try:
    regexp = r'masterPw="(.*)"'
    regx   = re.compile(regexp, flags)
    conf   = file("/etc/smbldap-tools/smbldap_bind.conf").read()
    password = regx.findall(conf)[0]
except Exception,e:
    print "cannot read password from /etc/smbldap-tools/smbldap_bind.conf"
    print e
    sys.exit(1)

try: 
    l = ldap.open("127.0.0.1")
    l.protocol_version = ldap.VERSION3
    username = "cn=admin,"+base
    l.simple_bind(username, password)

except ldap.LDAPError, e:
    print "Connection error", e

baseDN = "ou=Users,"+base
searchScope = ldap.SCOPE_ONELEVEL
searchFilter = "uid=*"
retrieveAttributes = ["uidNumber", "gidNumber", "homeDirectory"]

try:
    ldap_result_id = l.search(baseDN, searchScope, searchFilter, retrieveAttributes)
    result_set = []
    count = 0
    while 1:
	result_type, result_data = l.result(ldap_result_id, 0)
	if (result_data == []):
	    break
	else:
	    if result_type == ldap.RES_SEARCH_ENTRY:
		result_set.append(result_data)
#    print result_set
except ldap.LDAPError, e:
    print "Search error", e
    
if len(result_set) == 0:
    print "No Results"
    exit


for i in range(len(result_set)):
    for voce in result_set[i]:
	home = voce[1]['homeDirectory'][0]
	userid  = voce[1]['uidNumber'][0]
	groupid = voce[1]['gidNumber'][0]
	if (int(userid) >= 1000):
	    if (access(home, F_OK)):
#		print "Directory ", home, " already exists"
		continue
	    else:
                print "Creating ", home
		system('/bin/cp -a /etc/skel '+home)
		system('/bin/chown -R '+userid+'.'+groupid+' '+home)
                system('/bin/chmod 700 '+home)
#		chown(home,int(userid),int(groupid))
#		distutils.dir_util.copy_tree("/etc/skel", home)
